package container

import (
	"github.com/pkg/errors"
	corev1 "k8s.io/api/core/v1"
)

// GenerateConfigMap generates ConfigMap object as per the `Conf` struct passed
func GenerateContainer(c Conf) (obj *corev1.Container, err error) {
	var container corev1.Container
	var ports []corev1.ContainerPort
	var cmd []string
	var args []string
	var resources corev1.ResourceRequirements
	var volumeMounts []corev1.VolumeMount
	var readiness *corev1.Probe
	var liveness *corev1.Probe
	var envList []corev1.EnvVar

	if c.GenContainerPortsFunc != nil {
		ports, err = c.GenContainerPortsFunc(c.Instance)
		if err != nil {
			return nil, errors.Wrap(err, "failed to generate service ports")
		}
	}
	if c.GenContainerCmdFunc != nil {
		cmd, err = c.GenContainerCmdFunc(c.Instance)
		if err != nil {
			return nil, errors.Wrap(err, "failed to generate service ports")
		}
	}
	if c.GenContainerArgsFunc != nil {
		args, err = c.GenContainerArgsFunc(c.Instance)
		if err != nil {
			return nil, errors.Wrap(err, "failed to generate service ports")
		}
	}
	if c.GenContainerResourcesFunc != nil {
		resources, err = c.GenContainerResourcesFunc(c.Instance)
		if err != nil {
			return nil, errors.Wrap(err, "failed to generate service ports")
		}
	}
	if c.GenContainerVolumeMountsFunc != nil {
		volumeMounts, err = c.GenContainerVolumeMountsFunc(c.Instance)
		if err != nil {
			return nil, errors.Wrap(err, "failed to generate service ports")
		}
	}
	if c.GenContainerReadinessProbeFunc != nil {
		readiness, err = c.GenContainerReadinessProbeFunc(c.Instance)
		if err != nil {
			return nil, errors.Wrap(err, "failed to generate service ports")
		}
	}
	if c.GenContainerLivennessProbeFunc != nil {
		liveness, err = c.GenContainerLivennessProbeFunc(c.Instance)
		if err != nil {
			return nil, errors.Wrap(err, "failed to generate service ports")
		}
	}
	if c.GenContainerEnvVarFunc != nil {
		envList, err = c.GenContainerEnvVarFunc(c.Instance)
		if err != nil {
			return nil, errors.Wrap(err, "failed to generate service ports")
		}
	}
	container = corev1.Container{
		Name:            c.Name,
		Image:           c.Image,
		ImagePullPolicy: c.ImagePullPolicy,
		Args:            args,
		Command:         cmd,
		Env:             envList,
		Ports:           ports,
		VolumeMounts:    volumeMounts,
		LivenessProbe:   liveness,
		ReadinessProbe:  readiness,
		Resources:       resources,
	}

	return &container, err
}
