package container

import (
	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"

	corev1 "k8s.io/api/core/v1"
)

// GenConfigMapFunc defines a function which generates ConfigMap.
type GenContainerFunc func(Conf) (corev1.Container, error)

// GenServicePortsFunc defines a function which generates slice of
// ServicePort for Service
type GenContainerPortsFunc func(interfaces.Object) ([]corev1.ContainerPort, error)

type GenContainerCmdFunc func(interfaces.Object) ([]string, error)

type GenContainerArgsFunc func(interfaces.Object) ([]string, error)

type GenContainerResourcesFunc func(interfaces.Object) (corev1.ResourceRequirements, error)

type GenContainerVolumeMountsFunc func(interfaces.Object) ([]corev1.VolumeMount, error)

type GenContainerLivennessProbeFunc func(interfaces.Object) (*corev1.Probe, error)

type GenContainerReadinessProbeFunc func(interfaces.Object) (*corev1.Probe, error)

type GenContainerEnvVarFunc func(interfaces.Object) ([]corev1.EnvVar, error)

// Conf is used to pass parameters to functions in this package to
// perform operations on Configmap objects.
type Conf struct {
	// Instance is the Owner object which manages the Configmap
	Instance interfaces.Object
	// Name of the Configmap
	Name string
	// Namespace of the Configmap
	Namespace       string
	Image           string
	ImagePullPolicy corev1.PullPolicy
	GenContainerFunc
	GenContainerPortsFunc
	GenContainerCmdFunc
	GenContainerArgsFunc
	GenContainerResourcesFunc
	GenContainerVolumeMountsFunc
	GenContainerLivennessProbeFunc
	GenContainerReadinessProbeFunc
	GenContainerEnvVarFunc
}
