package job

import (
	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"
	"github.com/ankitrgadiya/operatorlib/pkg/meta"
	"github.com/ankitrgadiya/operatorlib/pkg/operation"
	batchv1 "k8s.io/api/batch/v1"

	pod "github.com/ankitrgadiya/operatorlib/pkg/pod"
)

// GenConfigMapFunc defines a function which generates ConfigMap.
type GenJobFunc func(Conf) (*batchv1.Job, error)
type GenParallelismFunc func(interfaces.Object) (*int32, error)
type GenCompletionsFunc func(interfaces.Object) (*int32, error)
type GenActiveDeadlineSecondsFunc func(interfaces.Object) (*int64, error)
type GenBackoffLimitFunc func(interfaces.Object) (*int32, error)

// type GenSelectorFunc func(interfaces.Object) (*metav1.LabelSelector, error)

// Conf is used to pass parameters to functions in this package to
// perform operations on Configmap objects.
type Conf struct {
	// Instance is the Owner object which manages the Configmap
	Instance interfaces.Object
	// Reconcile is the pointer to reconcile struct of owner object
	interfaces.Reconcile
	// Name of the Configmap
	Name string
	// Namespace of the Configmap
	Namespace string
	// GenLalebsFunc is used to generate labels for ObjectMeta
	meta.GenLabelsFunc
	// GenAnnotationsFunc is used to generate annotations for ObjectMeta
	meta.GenAnnotationsFunc
	// GenFinalizers is used to generate finalizers for ObjectMeta
	meta.GenFinalizersFunc
	// AppendLabels is used to determine if labels from Owner object
	// are to be inherited
	AppendLabels bool
	// OwnerReference is used to determine if owner reference needs to
	// be set on Configmap before creating it in cluster
	OwnerReference bool
	// MaybeUpdateFunc defines an update function with custom logic
	// for Configmap update
	operation.MaybeUpdateFunc
	// AfterCreateFunc hook is called after creating the Configmap
	operation.AfterCreateFunc
	// AfterUpdateFunc hook is called after updating the Configmap
	operation.AfterUpdateFunc
	// AfterDeleteFunc hook is called after deleting the Configmap
	operation.AfterDeleteFunc
	// GenConfigMapFunc defines a function to generate the Configmap
	// object. The package comes with default configmap generator
	// function which is used by operation functions. By specifying
	// this field, user can override the default function with a
	// custom one.
	//	EnvList            []corev1.EnvVar
	//	Size               int32
	ManualSelector bool
	GenJobFunc
	Pod pod.Conf
	GenParallelismFunc
	GenCompletionsFunc
	GenActiveDeadlineSecondsFunc
	GenBackoffLimitFunc
	//	GenSelectorFunc
}
