package pod

import (
	"github.com/ankitrgadiya/operatorlib/pkg/container"
	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"
	"github.com/ankitrgadiya/operatorlib/pkg/meta"
	"github.com/ankitrgadiya/operatorlib/pkg/operation"

	corev1 "k8s.io/api/core/v1"
)

// GenConfigMapFunc defines a function which generates ConfigMap.
type GenPodFunc func(Conf) (*corev1.PodTemplateSpec, error)

type GenContainersFunc []container.GenContainerFunc

// GenConfigMapFunc defines a function which generates ConfigMap.
type GenVolumesFunc func(interfaces.Object) ([]corev1.Volume, error)

type GenSecurityContextFunc func(interfaces.Object) (*corev1.PodSecurityContext, error)

// GenDataFunc defines a function which generates data (string map)
// for Configmap.
type GenInitContainersFunc func(interfaces.Object) (corev1.Container, error)
type GenRestartPolicyFunc func(interfaces.Object) (corev1.RestartPolicy, error)

// Conf is used to pass parameters to functions in this package to
// perform operations on Configmap objects.
type Conf struct {
	// Instance is the Owner object which manages the Configmap
	Instance interfaces.Object
	// Reconcile is the pointer to reconcile struct of owner object
	interfaces.Reconcile
	// Name of the Configmap
	Name string
	// Namespace of the Configmap
	Namespace string
	// GenLalebsFunc is used to generate labels for ObjectMeta
	meta.GenLabelsFunc
	// GenAnnotationsFunc is used to generate annotations for ObjectMeta
	meta.GenAnnotationsFunc
	// GenFinalizers is used to generate finalizers for ObjectMeta
	meta.GenFinalizersFunc
	// AppendLabels is used to determine if labels from Owner object
	// are to be inherited
	AppendLabels bool
	// OwnerReference is used to determine if owner reference needs to
	// be set on Configmap before creating it in cluster
	OwnerReference bool
	// MaybeUpdateFunc defines an update function with custom logic
	// for Configmap update
	operation.MaybeUpdateFunc
	// AfterCreateFunc hook is called after creating the Configmap
	operation.AfterCreateFunc
	// AfterUpdateFunc hook is called after updating the Configmap
	operation.AfterUpdateFunc
	// AfterDeleteFunc hook is called after deleting the Configmap
	operation.AfterDeleteFunc
	//	Volumes []corev1.Volume
	GenPodFunc
	Containers     []container.Conf
	InitContainers []container.Conf
	GenVolumesFunc
	GenSecurityContextFunc
	GenRestartPolicyFunc
}
